@@ -1,6 +1,6 @@
 ;;; copilot-balancer.el --- Parentheses balancer for Lisps  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2022-2026  copilot-emacs maintainers
+;; Copyright (C) 2022-2025  copilot-emacs maintainers
 
 ;; The MIT License (MIT)
 
@@ -36,15 +36,11 @@
 ;; If Copilot's completion quality for Lisp languages improves to the point
 ;; where balanced delimiters are reliably produced, this module can be removed
 ;; and `copilot-balancer-fix-completion' replaced with a simple pass-through.
-;; Users can disable balancing by setting `copilot-enable-parentheses-balancer'
-;; to nil.
 
 ;;; Code:
 
 (require 'cl-lib)
 
-(defvar copilot-enable-parentheses-balancer)
-
 (defvar copilot-balancer-lisp-modes '( emacs-lisp-mode
                                        lisp-mode
                                        lisp-interaction-mode
@@ -118,22 +114,13 @@ Uses SYNTAX-TABLE for parsing.  Returns a string of closing characters."
                        (scan-error (goto-char (point-max))))
                      (buffer-substring-no-properties end (point)))))
          (closers (copilot-balancer--compute-closers
-                   prefix trimmed suffix (syntax-table)))
-         (replaced-closers
-          (when (< start end)
-            (let ((ppss (syntax-ppss start)))
-              (when (or (nth 3 ppss) (nth 4 ppss))
-                (let* ((replaced (buffer-substring-no-properties start end))
-                       (replaced-trimmed
-                        (copilot-balancer-trim-closing-pairs-at-end replaced)))
-                  (substring replaced (length replaced-trimmed))))))))
-    (list start end (concat trimmed closers (or replaced-closers "")))))
+                   prefix trimmed suffix (syntax-table))))
+    (list start end (concat trimmed closers))))
 
 (defun copilot-balancer-fix-completion (start end completion)
   "Fix COMPLETION from START to END."
   (cond
-   ((and copilot-enable-parentheses-balancer
-         (apply #'derived-mode-p copilot-balancer-lisp-modes))
+   ((apply #'derived-mode-p copilot-balancer-lisp-modes)
     (copilot-balancer--fix-lisp start end completion))
    (t (list start end completion))))
 
